@@ -44,7 +44,7 @@ from intentkit.core.prompt import (
 from intentkit.core.skill import skill_store
 from intentkit.models.agent import Agent, AgentTable
 from intentkit.models.agent_data import AgentData, AgentQuota
-from intentkit.models.app_setting import AppSetting
+from intentkit.models.app_setting import AppSetting, SystemMessageType
 from intentkit.models.chat import (
     AuthorType,
     ChatMessage,
@@ -298,16 +298,14 @@ async def stream_agent(message: ChatMessageCreate):
         if agent.fee_percentage and agent.fee_percentage > 100:
             owner = await User.get(agent.owner)
             if owner and agent.fee_percentage > 100 + owner.nft_count * 10:
-                error_message_create = ChatMessageCreate(
-                    id=str(XID()),
+                error_message_create = await ChatMessageCreate.from_system_message(
+                    SystemMessageType.SERVICE_FEE_ERROR,
                     agent_id=input.agent_id,
                     chat_id=input.chat_id,
                     user_id=input.user_id,
                     author_id=input.agent_id,
-                    author_type=AuthorType.SYSTEM,
                     thread_type=input.author_type,
                     reply_to=input.id,
-                    message="If you are the owner of this agent, please Update the Service Fee % to be in compliance with the Nation guidelines (Max 100% + 10% per Nation Pass NFT held)",
                     time_cost=time.perf_counter() - start,
                 )
                 error_message = await error_message_create.save()
@@ -336,16 +334,14 @@ async def stream_agent(message: ChatMessageCreate):
             abuse_check = False
         if abuse_check and payer != agent.owner and user_account.free_credits > 0:
             if quota and quota.free_income_daily > 24000:
-                error_message_create = ChatMessageCreate(
-                    id=str(XID()),
+                error_message_create = await ChatMessageCreate.from_system_message(
+                    SystemMessageType.DAILY_USAGE_LIMIT_EXCEEDED,
                     agent_id=input.agent_id,
                     chat_id=input.chat_id,
                     user_id=input.user_id,
                     author_id=input.agent_id,
-                    author_type=AuthorType.SYSTEM,
                     thread_type=input.author_type,
                     reply_to=input.id,
-                    message="This Agent has reached its free CAP income limit for today! Start using paid CAPs or wait until this limit expires in less than 24 hours.",
                     time_cost=time.perf_counter() - start,
                 )
                 error_message = await error_message_create.save()
@@ -356,16 +352,14 @@ async def stream_agent(message: ChatMessageCreate):
         if quota and quota.avg_action_cost > 0:
             avg_count = quota.avg_action_cost
         if not user_account.has_sufficient_credits(avg_count):
-            error_message_create = ChatMessageCreate(
-                id=str(XID()),
+            error_message_create = await ChatMessageCreate.from_system_message(
+                SystemMessageType.INSUFFICIENT_BALANCE,
                 agent_id=input.agent_id,
                 chat_id=input.chat_id,
                 user_id=input.user_id,
                 author_id=input.agent_id,
-                author_type=AuthorType.SYSTEM,
                 thread_type=input.author_type,
                 reply_to=input.id,
-                message="Insufficient balance.",
                 time_cost=time.perf_counter() - start,
             )
             error_message = await error_message_create.save()
@@ -723,17 +717,17 @@ async def stream_agent(message: ChatMessageCreate):
                                 cold_start_cost = 0
                             post_model_message = await post_model_message_create.save()
                             yield post_model_message
-                        error_message_create = ChatMessageCreate(
-                            id=str(XID()),
-                            agent_id=input.agent_id,
-                            chat_id=input.chat_id,
-                            user_id=input.user_id,
-                            author_id=input.agent_id,
-                            author_type=AuthorType.SYSTEM,
-                            thread_type=input.author_type,
-                            reply_to=input.id,
-                            message="Insufficient balance.",
-                            time_cost=0,
+                        error_message_create = (
+                            await ChatMessageCreate.from_system_message(
+                                SystemMessageType.INSUFFICIENT_BALANCE,
+                                agent_id=input.agent_id,
+                                chat_id=input.chat_id,
+                                user_id=input.user_id,
+                                author_id=input.agent_id,
+                                thread_type=input.author_type,
+                                reply_to=input.id,
+                                time_cost=0,
+                            )
                         )
                         error_message = await error_message_create.save()
                         yield error_message
@@ -749,16 +743,14 @@ async def stream_agent(message: ChatMessageCreate):
             f"failed to execute agent: {str(e)}\n{error_traceback}",
             extra={"thread_id": thread_id},
         )
-        error_message_create = ChatMessageCreate(
-            id=str(XID()),
+        error_message_create = await ChatMessageCreate.from_system_message(
+            SystemMessageType.AGENT_INTERNAL_ERROR,
             agent_id=input.agent_id,
             chat_id=input.chat_id,
             user_id=input.user_id,
             author_id=input.agent_id,
-            author_type=AuthorType.SYSTEM,
             thread_type=input.author_type,
             reply_to=input.id,
-            message="Agent Internal Error",
             time_cost=time.perf_counter() - start,
         )
         error_message = await error_message_create.save()
@@ -770,16 +762,14 @@ async def stream_agent(message: ChatMessageCreate):
             f"reached recursion limit: {str(e)}\n{error_traceback}",
             extra={"thread_id": thread_id, "agent_id": input.agent_id},
         )
-        error_message_create = ChatMessageCreate(
-            id=str(XID()),
+        error_message_create = await ChatMessageCreate.from_system_message(
+            SystemMessageType.STEP_LIMIT_EXCEEDED,
             agent_id=input.agent_id,
             chat_id=input.chat_id,
             user_id=input.user_id,
             author_id=input.agent_id,
-            author_type=AuthorType.SYSTEM,
             thread_type=input.author_type,
             reply_to=input.id,
-            message="Step Limit Error",
             time_cost=time.perf_counter() - start,
         )
         error_message = await error_message_create.save()
@@ -791,16 +781,14 @@ async def stream_agent(message: ChatMessageCreate):
             f"failed to execute agent: {str(e)}\n{error_traceback}",
             extra={"thread_id": thread_id, "agent_id": input.agent_id},
         )
-        error_message_create = ChatMessageCreate(
-            id=str(XID()),
+        error_message_create = await ChatMessageCreate.from_system_message(
+            SystemMessageType.AGENT_INTERNAL_ERROR,
             agent_id=input.agent_id,
             chat_id=input.chat_id,
             user_id=input.user_id,
             author_id=input.agent_id,
-            author_type=AuthorType.SYSTEM,
             thread_type=input.author_type,
             reply_to=input.id,
-            message="Agent Internal Error",
             time_cost=time.perf_counter() - start,
         )
         error_message = await error_message_create.save()
